/*
 * $Id: fp_jac.c,v 1.3 1997/05/07 15:24:54 veran Exp $
 */

/* ROUTINE TO COMPUTE JACOBIAN MATRIX
   FOR FIXED POINT ROUTINES

   This will be done by finite difference methods
   or by formula.

   */
#include <stdio.h>

#include "constants.h"
#include "math_utils.h"
#include "iterate.h"
#include "fixed.h"


/* procedure to compute a jacobian matrix of F(x)
	where F(x) = f^r(x) - x  for a mapping
	      F(x) = f(x)        for a vector field	

	r is the period of the fixed point

	the point is taken to be x1
	the finite difference step is x2
	the jacobian is put in jacobian
	the function evaluation F(x1) is put in fx

	set fd_flag to be non-zero to force finite difference jacobian evaluation
*/

int
fp_jac( struct Fixpt_DataS* fp, int fd_flag)
{
  int i, status = 0, var_dim  = fp->prop_cntl.ph_space_dim - 1;

  if (fp->prop_cntl.mapping_flag) 
    {	  
      /* it's a mapping */
      if (fp->prop_cntl.dfdx && fd_flag == 0) 
	{
	  /* jacobian is given */
	  status = dfdx(fp->jacobian, var_dim, NULL, fp->x1, fp->prop_cntl.parameters, 
			fp->fp_map_period, fp->prop_cntl.manifold, 
			fp->prop_cntl.mapping_flag, fp->prop_cntl.function, fp->prop_cntl.dfdx,
			ANALYTIC, fp->prop_cntl.workspace);
	}
      else 
	{
	  /* compute finite difference jacobian */
	  status = dfdx(fp->jacobian, var_dim, fp->x2, fp->x1, fp->prop_cntl.parameters, 
			fp->fp_map_period, fp->prop_cntl.manifold, 
			fp->prop_cntl.mapping_flag, fp->prop_cntl.function, fp->prop_cntl.dfdx,
			CEN_DIFF, fp->prop_cntl.workspace);
	}
      iter_forw(fp->prop_cntl.function, fp->fp_map_period, fp->fx, fp->x1, fp->prop_cntl.parameters, 
		fp->prop_cntl.ph_space_dim, 1.0, fp->prop_cntl.workspace, fp->prop_cntl.manifold);
      lift(var_dim, fp->x1, fp->fx, fp->fx, fp->prop_cntl.manifold);
      for (i=0; i<var_dim; i++) 
	{
	  fp->jacobian[i][i] -= 1.0;
	  fp->fx[i] -= fp->x1[i];
	}
    }
  else 
    {
      /* vector field */
      if (fp->prop_cntl.dfdx && fd_flag == 0) 
	{	  
	  /* jacobian is given */
	  status = dfdx(fp->jacobian, var_dim, NULL, fp->x1, fp->prop_cntl.parameters, 
			0, fp->prop_cntl.manifold, 
			fp->prop_cntl.mapping_flag, fp->prop_cntl.function, fp->prop_cntl.dfdx,
			ANALYTIC, fp->prop_cntl.workspace);
	}
      else 
	{
	  /* compute finite difference jacobian */
	  status = dfdx(fp->jacobian, var_dim, fp->x2, fp->x1, fp->prop_cntl.parameters, 
			0, fp->prop_cntl.manifold, 
			fp->prop_cntl.mapping_flag, fp->prop_cntl.function, fp->prop_cntl.dfdx,
			CEN_DIFF, fp->prop_cntl.workspace);
	}
      fp->prop_cntl.function(fp->fx, fp->x1, fp->prop_cntl.parameters);
    }
  return(status);
}


