/*
 * $Id: flow.c,v 1.3 1997/05/08 18:45:14 veran Exp $
 */

/*
 * flow.c
 */
#include <math.h>

#include "utilities.h"
#include "constants.h"
#include "ring_local.h"
#include "man2d_local.h"

int
flow_man(struct Man2d_DataS *s)
{
  int i, status = 0, plot_counter = 0;
  int normal_vf();
  ring r, rp;
  double sep_dist, deltat = s->stepsize*s->stability;
  
  system_mess_proc(0,"Flow approximation of twod manifolds.");

  r = linear_approx(s);

/*   normal_set_vf(s->vf, s->v_dim-1); */
  for (i=0; i<s->steps; i++)
    {
      /* extend this ring */
      ring_extend(r);
      
      /* fill in values by using rk steps */
      rp = r;
      do {
	rk4(rp->out->x, rp->x, s->p, deltat, s->v_dim-1, s->vf, s->work);
	rp = rp->right;
      } while (rp != r);

      /* shift pointer to outside ring */
      r = r->out;
      if (i<10)
	{
	  sep_dist = 2*avg_dist(r);
	  if (sep_dist>s->dist) sep_dist = s->dist;
	  simple_interp(r, sep_dist);
	}
      else
	{
	  simple_interp(r, s->dist);
	}

      /* plot ring? */
      if (plot_counter-- == 0)
	{
	  ring_plot(r,s->p,s->color,s->symbol);
	  plot_counter = s->skip;
	  
	  /* check for interrupts when rings are plotted */
	  if (interrupt()) i = s->steps;
	}

      /* if previous ring was not plotted then excise the ring */
      if ((plot_counter != s->skip-1) && (s->skip>0))
	{
	  ring_excise(rp);
	}
    }

  /* plot last ring, if not plotted already */
  if (plot_counter != s->skip)
    ring_plot(r,s->p,s->color,s->symbol);

  ring_save(r,s->memory,s->p,s->color,s->symbol,s->man_type,0);
  ring_free(r);
  return status;
}


