/*
 * $Id: linear_approx.c,v 1.3 1997/05/08 18:45:14 veran Exp $
 */

/*
 * linear_approx.c
 */
#include <math.h>
#include <stdio.h>

#include "constants.h"
#include "math_utils.h"
#include "man2d_local.h"


/*
 * produced linear approximation to manifold
 */

ring
linear_approx(struct Man2d_DataS *s)
{
  ring r, r1;
  int i, j, var_dim = s->v_dim-1;
  double norm1, norm2, *evec1, *evec2, dotprod;
  double t, cost, sint, incr = 2*PI/s->points;

  r = ring_create(s->points, s->v_dim);
  evec1 = dvector(0, var_dim-1);
  evec2 = dvector(0, var_dim-1);

  fprintf(stdout,"Eigenvector: %g+%gI  (", s->eval[0][s->ev1],
	  s->eval[1][s->ev1]);
  for (i=0; i<var_dim; i++)
    fprintf(stdout,"%g ", s->evectors[s->ev1][i]);
  fprintf(stdout,")\nEigenvector: %g+%gI  (", s->eval[0][s->ev2],
	  s->eval[1][s->ev2]);
  for (i=0; i<var_dim; i++)
    fprintf(stdout,"%g ", s->evectors[s->ev2][i]);
  fprintf(stdout,")\n");

  /* copy and compute the norm of the first eigenvector */
  norm1 = 0.0;
  for (i = 0; i<var_dim; i++)
    {
      evec1[i] = s->evectors[s->ev1][i];
      norm1 += evec1[i]*evec1[i];
    }
  norm1 = sqrt(norm1);

  /* copy the second ev, normalize the first, and take the dotproduct */
  dotprod = 0.0;
  for (i=0; i<var_dim; i++)
    {
      evec2[i] = s->evectors[s->ev2][i];
      evec1[i] = evec1[i]/norm1;
      dotprod += evec1[i]*evec2[i];
    }

  /* subtract the projection of the second onto the first, and compute norm */
  norm2 = 0.0;
  for (i=0; i<var_dim; i++)
    {
      evec2[i] -= dotprod*evec1[i];
      norm2 += evec2[i]*evec2[i];
    }
  norm2 = sqrt(norm2);

  /* normalize the result */
  for (i=0; i<var_dim; i++)
    {
      evec2[i] /= norm2;
    }

  t = 0.0;
  r1 = r;
  for(i=0; i<s->points; i++)
    {
      cost = cos(t)*s->eigen_dist;
      sint = sin(t)*s->eigen_dist;
      for (j=0; j<var_dim; j++)
	r1->x[j] = cost*evec1[j] + sint*evec2[j]+s->x1[j];
      r1->x[var_dim] = s->x1[var_dim]; /* copy time */
      t+= incr;
      r1 = r1->left;
    }

  free_dvector(evec1, 0, var_dim-1);
  free_dvector(evec2, 0, var_dim-1);

  return r;
}
