/*
 * $Id: man2d_go.c,v 1.3 1997/05/08 18:45:14 veran Exp $
 */

/*
 * man2d_go.c
 *
 */
#include "pm.h"
#include "constants.h"
#include "utilities.h"
#include "eigen.h"
#include "defaults.h"
#include "man2d_local.h"

void
man2d_go(void)
{
  int status = 0, num = 0;
  struct Man2d_DataS s;
  int *extra_iparams, n_extra_iparams;
  double *points, *params;
  int var_dim;
  int i, j, k, n_unstable, n_stable;

  if (*((int *) pm(GET, "Model.Mapping_Flag", NULL)))
    {
      system_mess_proc(1, "Sorry, these algorithms are only for vector fields.");
      return;
    }
  if (*((int *) pm(GET, "Model.Varb_Dim", NULL)) < 4)  /* v_dim includes time */
    {
      system_mess_proc(1, "Not much point computing 2-d manifolds in a 2-d system!");
    }
  
  if (man2d_load(&s) == -1) 
    {
      system_mess_proc(1,"man2d_go: Memory alloc error.");
      return;
    }

  var_dim = s.v_dim-1;  /* don't include time */

  system_mess_proc(0,"Computing 2-d manifolds of fixed points.");
  reset_interrupt();
  
  /* search memory object for fixed points */
  if (memory_reset_read(s.memory) == 0)
    {
      while (memory_read_next_flow(s.memory, NULL, NULL, NULL, 
				   &extra_iparams, &n_extra_iparams) == 0 
	     && status>=0)
	{
	  while (memory_read_next_traj(s.memory, NULL,
				       NULL, NULL) == 0 && 
		 status >= 0)
	    {
	      if (extra_iparams[0] == SADDLE)
		{
		  while (memory_read_next_point(s.memory, &points, &params, 
						NULL, NULL, NULL) == 0 && status >= 0)
		    {
		      for (j=0; j<s.v_dim; j++) 
			{
			  s.x1[j] = points[j];
			  s.x2[j] = s.fd_step;
			}
		      for (j=0; j<s.p_dim; j++)
			s.p[j] = params[j];
		      compute_jacobian(s.jacobian, s.x1, s.p, s.x2,
				       s.v_dim, s.work);
		      /* DEBUG - print out jacobian 
			 fprintf(stdout,"Jacobian: ");
			 for (j=0; j<var_dim; j++)
			 {
			 for (k=0; k<var_dim; k++)
			 fprintf(stdout,"%lf ",s.jacobian[j][k]);
			 fprintf(stdout,"\n          ");
			 }
			 fprintf(stdout,"\n");  */
		      
		      /* use converted Fortran routines to compute
			 evals and evectors */
		      for (j=0; j<var_dim; j++)
			for (k=0; k<var_dim; k++)
			  s.jact[j+1][k+1] = s.jacobian[j][k];
		      status = rg(var_dim,var_dim,s.jact,s.eval[0]-1,
				  s.eval[1]-1,1,s.evectors,s.indx,s.x3);
		      for (j=0; j<var_dim; j++)
			for (k=0; k<var_dim; k++)
			  s.evectors[j][k] = s.evectors[j+1][k+1];
		      
		      n_stable = 0;
		      n_unstable = 0;
		      for(j=0; j<var_dim; j++)
			{
			  if (s.eval[0][j] > 0.0) 
			    {
			      n_unstable++;
			      s.indx[j] = 1;
			    }
			  else if (s.eval[0][j] < 0.0)
			    {
			      n_stable++;
			      s.indx[j] = -1;
			    }
			  else s.indx[j] = 0;
			}
		      while ((n_stable == 2) || (n_unstable==2))
			{
			  /* now do it! */
			  num++;
			  if (n_stable == 2)
			    {
			      s.stability = -1;
			      s.ev1 = -1;
			      i = 0;
			      do {
				if (s.indx[i] == -1) s.ev1 = i;
				i++;
			      } while (s.ev1==-1);
			      s.ev2 = -1;
			      do {
				if (s.indx[i] == -1) s.ev2 = i;
				i++;
			      } while (s.ev2 == -1);
			      s.color = STAB_MAN_COLOR;
			      s.man_type = STAB_MAN2D;
			      n_stable = 0; /* this one is done */
			      fprintf(stderr," stable 2d manifold: %f+%fI %f+%fI\n", 
				      s.eval[0][s.ev1], s.eval[1][s.ev1], 
				      s.eval[0][s.ev2], s.eval[1][s.ev2]);
			    }
			  else
			    {
			      s.stability = 1;
			      s.ev1 = -1;
			      i = 0;
			      do {
				if (s.indx[i] == 1) s.ev1 = i;
				i++;
			      } while (s.ev1==-1);
			      s.ev2 = -1;
			      do {
				if (s.indx[i] == 1) s.ev2 = i;
				i++;
			      } while (s.ev2 == -1);
			      s.color = UNSTAB_MAN_COLOR;
			      s.man_type = UNSTAB_MAN2D;
			      n_unstable = 0; /* this one is done too! */
			      fprintf(stderr," unstable 2d manifold: %f+%fI %f+%fI\n", 
				      s.eval[0][s.ev1], s.eval[1][s.ev1], 
				      s.eval[0][s.ev2], s.eval[1][s.ev2]);
			    }
			  switch (s.algorithm) 
			    {
			    case 0:
			      status = simple(&s);
			      break;
			    case 1:
			      status = flow_man(&s);
			      break;
			    case 2:
			      system_mess_proc(0,"Geodesic algorithm - not yet implemented.");
			      break;
			    case 3:
			      system_mess_proc(0,"Two point BV algorithm - not yet implemented.");
			      break;
			    default:
			      break;
			    }
			}
		    }
		}
	    }
	}
    }

  if (num==0)
    {
      system_mess_proc(1,"man2d_go: No 2-d manifolds to draw. (ie, no saddles in fixed point memory)");
    }

  release_man2d(&s);

  pm(EXEC, "Memory.Count", NULL);
  return;
}
