/*
 * $Id: man2d_load.c,v 1.3 1997/05/08 18:45:14 veran Exp $
 */

/*
 * man2d_load.c
 */

#include "pm.h"
#include "constants.h"
#include "math_utils.h"
#include "symbols.h"
#include "man2d_local.h"

int
man2d_load(struct Man2d_DataS *s)
{
  int status = NO_ERROR;

  /* load info from the panels */
  s->algorithm = *((int *) pm(GET, "Man2d.Algorithm", NULL));
  s->eigen_dist = *((double *) pm(GET, "Man2d.Eigen_Dist", NULL));
  s->stepsize = *((double *) pm(GET, "Man2d.Stepsize", NULL));
  s->points = *((int *) pm(GET, "Man2d.Points", NULL));
  s->steps = *((int *) pm(GET, "Man2d.Steps", NULL));
  s->memory = (memory) pm(GET, "Memory.Fixed", NULL);
  s->fd_step = *((double *) pm(GET, "Fixed.FD_Step", NULL));
  s->dist = *((double *) pm(GET, "Man2d.Dist", NULL));
  s->skip = *((int *) pm(GET, "Man2d.Skip", NULL));

  /* load info about model */
  s->v_dim = *((int *) pm(GET, "Model.Varb_Dim", NULL));
  s->p_dim = *((int *) pm(GET, "Model.Param_Dim", NULL));
  s->vf = (int (*)()) pm(GET, "Model.DS_Def", NULL);
  
  s->x1 = dvector(0, s->v_dim-1);
  s->x2 = dvector(0, s->v_dim-1);
  s->x3 = dvector(0, s->v_dim-1);
  s->f = dvector(0, s->v_dim-1);
  s->p = dvector(0, s->p_dim-1);
  s->work = dvector(0, 10*s->v_dim-1);
  s->jacobian = dmatrix(0, s->v_dim-1, 0, s->v_dim-1);
  s->jact = dmatrix(0, s->v_dim-1, 0, s->v_dim-1);
  s->eval = dmatrix(0,1,0,s->v_dim-1);
  s->evectors = dmatrix(0,s->v_dim-1, 0, s->v_dim-1);
  s->indx = ivector(0, s->v_dim-1);
  s->symbol = SMALL_POINT;
  s->color = 0;

  return status;

}

int
release_man2d(struct Man2d_DataS *s)
{
  int status = NO_ERROR;

  free_dvector(s->x1, 0, s->v_dim-1);
  free_dvector(s->x2, 0, s->v_dim-1);
  free_dvector(s->x3, 0, s->v_dim-1);
  free_dvector(s->f, 0, s->v_dim-1);
  free_dvector(s->p, 0, s->p_dim-1);
  free_dvector(s->work, 0, 10*s->v_dim-1);
  free_dmatrix(s->jacobian, 0, s->v_dim-1, 0, s->v_dim-1);
  free_dmatrix(s->jact, 0, s->v_dim-1, 0, s->v_dim-1);
  free_dmatrix(s->eval, 0, 1, 0, s->v_dim-1);
  free_dmatrix(s->evectors, 0, s->v_dim-1, 0, s->v_dim-1);
  free_ivector(s->indx, 0, s->v_dim-1);

  return status;
}

