/*
 * $Id: simple.c,v 1.3 1997/05/08 18:45:14 veran Exp $
 */

/*
 * simple.c
 */
#include <math.h>

#include "utilities.h"
#include "constants.h"
#include "ring_local.h"
#include "man2d_local.h"

int
simple(struct Man2d_DataS *s)
{
  int i, status = 0, plot_counter = 0;
  int normal_vf();
  ring r, rp;
  double sep_dist, deltat = s->stepsize*s->stability;
  
  system_mess_proc(0,"Simple algorithm for computing twod manifolds.");

  r = linear_approx(s);

  normal_set_vf(s->vf, s->v_dim-1);
  for (i=0; i<s->steps; i++)
    {
      /* extend this ring */
      ring_extend(r);
      
      /* fill in values by using rk steps */
      rp = r;
      do {
	rk4(rp->out->x, rp->x, s->p, deltat, s->v_dim-1, normal_vf, s->work);
	rp = rp->right;
      } while (rp != r);

      /* shift pointer to outside ring */
      r = r->out;
      if (i<10)
	{
	  sep_dist = 2*avg_dist(r);
	  if (sep_dist>s->dist) sep_dist = s->dist;
	  simple_interp(r, sep_dist);
	}
      else
	{
	  simple_interp(r, s->dist);
	}

      /* plot ring? */
      if (plot_counter-- == 0)
	{
	  ring_plot(r,s->p,s->color,s->symbol);
	  plot_counter = s->skip;
	  
	  /* check for interrupts when rings are plotted */
	  if (interrupt()) i = s->steps;
	}

      /* if previous ring was not plotted then excise the ring */
      if ((plot_counter != s->skip-1) && (s->skip>0))
	{
	  ring_excise(rp);
	}
    }

  /* plot last ring, if not plotted already */
  if (plot_counter != s->skip)
    ring_plot(r,s->p,s->color,s->symbol);

  ring_save(r,s->memory,s->p,s->color,s->symbol,s->man_type,0);
  ring_free(r);
  return status;
}

void
simple_interp(ring r, double dist)
{
  ring rp, new;
  int i, added;
  double delta, d, ds = dist*dist;

  rp = r;
  do {
    added = 0;
    d = 0.0;
    for (i=0; i<r->x_dim-1; i++)
      {
	delta = rp->x[i]-rp->right->x[i];
	d+= delta*delta;
      }
    if (d > ds)
      {
	/* insert a new point */
	added = 1;
	new = ring_create(1,rp->x_dim);
	new->index = -1;
	new->left = rp;
	new->right = rp->right;
	rp->right->left = new;
	rp->right = new;
	for (i=0; i<rp->x_dim; i++)
	  new->x[i] = (new->left->x[i]+new->right->x[i])/2.0;
      }
    else
      rp = rp->right;
  } while ((rp != r) || added);
}

double
avg_dist(ring r)
{
  int i, n=0, dim = r->x_dim;
  double d = 0, dp, delta;
  ring rp;

  rp = r;
  do {
    dp = 0;
    for (i=0; i<dim-1; i++)
      {
	delta = rp->x[i] - rp->right->x[i];
	dp += delta*delta;
      }
    d += sqrt(dp);
    n++;
    rp = rp->right;
  } while (rp != r);

  return(d/n);
}
