/*
 * $Id: lubksb.c,v 1.2 1997/01/07 16:42:45 veran Exp $
 */

/*
lubksb() performs LU back substitution.  This function is taken from NUMERICAL 
	RECIPES, Press, et al, p. 44.
	This solves a set of n linear equations a.x = b.  Here a[0,..n-1][0,..n-1]
	(Note: our indices are different than those in Press et al!)
	is input, not as the matrix a, but as its LU decomp, performed by ludcmp.
	indx[0,..n-1] is input as the permutation vector returned by ludcmp.
	b[0,...n-1] is input as the RHS vector and upon return it contains the 
	solution vector x.
*/

int
lubksb(a,n,indx,b)
double **a,b[];
int n,indx[];
{
	int i,ii= -1,ip,j;
	double sum;

	for(i=0;i<n;i++){
		ip = indx[i];
		sum = b[ip];
		b[ip] = b[i];
		if(ii >= 0)
			for(j=ii;j<=i-1;j++) sum -= a[i][j] * b[j];
		else if (sum) ii = i;
		b[i] = sum;
	}
	for(i=n-1;i>=0;i--){
		sum = b[i];
		for(j=i+1;j<n;j++) sum -= a[i][j] * b[j];
		b[i] = sum/a[i][i];
	}
	return(0);
}
