/*
 * $Id: tcl_utils.c,v 1.2 1997/01/07 17:04:08 veran Exp $
 */

/*
 * tcl_utils.c
 *
 * prepend_to_auto_path()
 *
 */
#include <tcl.h>
#include <utilities.h>

extern Tcl_Interp *interp;

/*
 * autopath is the directory list variable which tcl uses to 
 * look for unknown commands
 */
int
prepend_to_auto_path(const char *path)
{
  char script[256];

  sprintf(script, "set auto_path [linsert $auto_path 0 %s]", path);
  return tcl_script(script);
}


/*
 * execute script in tcl
 *
 */
int
tcl_script(char *script)
{
  int status;

  if ( (status = Tcl_GlobalEval(interp,script)) == TCL_ERROR)
    {
      fprintf(stderr, "tcl_script - error executing: %s\n", script);
      Tcl_GlobalEval(interp,"puts stderr $errorInfo");
    }

  return status;
}
 

/*
 * returns ClientData for a tcl object
 */
ClientData
get_ClientData(char *obj)
{
  Tcl_CmdInfo info;

  if (Tcl_GetCommandInfo(interp, obj, &info))
    return info.clientData;

  fprintf(stderr,"get_ClientData - bad object name %s\n",obj);
  return NULL;
}

/* we want to bypass Tcl_Eval for speed in some case, so store some cmd info */
static Tcl_CmdInfo update_info;

/*
 * tcl_update
 *
 * updates the tcl windows
 */
void
tcl_update()
{
  static char update[] = "update";
  char *argv[1];
  int argc=1;

  argv[0] = update;

  (*update_info.proc)(update_info.clientData, interp, argc, argv);
}

/*
 * get_updateinfo
 */
void
setup_tcl_update()
{
  static char update[] = "update";

  /* get info on some commands for optimization */
  Tcl_GetCommandInfo(interp, update, &update_info);
}

