/*
 * $Id: user_util.c,v 1.2 1997/01/07 17:04:08 veran Exp $
 */

/*
 * user_util.c
 *
 *  get_user_info() returns the user's full name and any other personal info
 * 	contained in the 5th field of the record in /etc/passwd which begins
 *	with their login name.  If L_Name is the loginname of the user then 
 * 	the information returned is the same as the info gotten by the shell
 *	command   
 *	/usr/bin/awk '{FS=":"} {if ($1 ~ /^L_Name/) {print $5}}' /etc/passwd
 *  From the GETLOGIN(3) man page:
 *	"The correct  procedure for determining the login name is to call
 *	cuserid, or to call getlogin() and, if  it  fails,  to  call
 *	getpwuid(getuid())."
 *  Because of the controversy over cuserid() (see CUSERID(3V)) and whether
 *	it will continue to be part of the standard library, we have 
 *	chosen to not use cuserid().
 */

#include <sys/types.h>
#include <pwd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <pwd.h>

#include <utilities.h>

char *
get_user_info()
{
  char		*name;
  struct passwd 	*p;

  if( (name = getlogin()) )
    p = getpwnam(name);
  else 
    p = getpwuid(getuid());
  return(p->pw_gecos);
}
